enterArray(aStr, iCol)
{
    walk(iCol, aStr, setCellText);
}

enterNodes(nodes, iCol)
{
    process(iRow, iCol, xn){
        setCellText(iRow, iCol, xn.InnerText);
    }
    walk(iCol, nodes, process);
}

processRange(sCoordsFrom, sCoordsTo, cb)
{
    result = null;
    iRowFrom = -1; iColFrom = -1;
    CellUtil.ParseCoordinates(sCoordsFrom, iRowFrom, iColFrom);
    iRowTo = -1; iColTo = -1;
    CellUtil.ParseCoordinates(sCoordsTo, iRowTo, iColTo);
    Check.ValueSmallerOrEqual(iRowFrom, iRowTo, "Bad range.");
    Check.ValueSmallerOrEqual(iColFrom, iColTo, "Bad range.");
    for(iCol = iColFrom; Flow.IsSmallerOrEqual(iCol, iColTo); iCol = Mat.Increment(iCol)){
        for(iRow = iRowFrom; Flow.IsSmallerOrEqual(iRow, iRowTo); iRow = Mat.Increment(iRow)){
            result = cb(iRow, iCol, result, Cell(iRow, iCol));
        }
    }
    result;
}

rangeMax(sCoordsFrom, sCoordsTo)
{
    process(iRow, iCol, result, value){
        if(Flow.IsNothing(result)){
            result = double.MinValue;
        }
        if(CellUtil.IsNumber(value)){
            value = Convert.ToDouble(value);
            if(Flow.IsGreater(value, result)){
                result = value;
            }
        }else{
            Check.Fail("#NaN");
        }
        result;
    }
    processRange(sCoordsFrom, sCoordsTo, process);
}

rangeMin(sCoordsFrom, sCoordsTo)
{
    process(iRow, iCol, result, value){
        if(Flow.IsNothing(result)){
            result = double.MaxValue;
        }
        if(CellUtil.IsNumber(value)){
            value = Convert.ToDouble(value);
            if(Flow.IsSmaller(value, result)){
                result = value;
            }
        }else{
            Check.Fail("#NaN");
        }
        result;
    }
    processRange(sCoordsFrom, sCoordsTo, process);
}

readUrl(sUrl)
{
    resp = System.Net.WebRequest.Create(sUrl).GetResponse();
    rdr = new System.IO.StreamReader(resp.GetResponseStream());
    s = rdr.ReadToEnd();
    rdr.Close();
    s;
}

setCellText(iRow, iCol, s)
{
    if(Flow.IsString(s)){
        SetCellExpression(iRow, iCol, string.Concat('\x22', s, '\x22'));
    }
}

splitString(s)
{
    s.Split(Parse.GetStandardDelimiters());
}

walk(context, col, cb)
{
    i = 0;
    foreach(obj in col){
        cb(i, context, obj);
        i = Mat.Increment(i);
    }
}

xpath(sUrl, sXPathExpr)
{
    xd = Dev.ReadXmlFile(sUrl);
    xd.SelectNodes(sXPathExpr);
}
